/*
 *  (c) 2011 Steffen Bondorf, disco | Distributed Computer Systems Lab
 *                                  University of Kaiserslautern, Germany
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.demos;

import java.util.Iterator;
import java.util.List;

import unikl.disco.curves.Curve;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.PmooSfaAnalysis;

import unikl.disco.network.Edge;
import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Vertex;

/**
 * 
 * @author Steffen Bondorf
 */
public class Demo2
{
	public Demo2() {}

	@SuppressWarnings("unused")
	public void run()
	{
		Curve service_curve = Curve.createRateLatency( 0.01, 10.0e6 );
		Curve max_service_curve = Curve.createRateLatency( 0.001, 100.0e6 );
		
		Network network = new Network();
		
		try {
			Vertex v0 = network.addVertex( service_curve, max_service_curve );
			v0.setUseGamma( false );
			v0.setUseExtraGamma( false );
			
			Vertex v1 = network.addVertex( service_curve, max_service_curve );
			v1.setUseGamma( false );
			v1.setUseExtraGamma( false );
		
			Edge e_0_1 = network.addEdge( v0, v1 );
	
			Curve arrival_curve = Curve.createTokenBucket( 0.1 * 0.1e6, 0.1e6 );
			
			Flow f0 = network.addFlow( arrival_curve, v1 );

			List<Edge> path = network.getShortestPath( v0, v1 );
			Flow f1 = network.addFlow( arrival_curve, path );
			
			Iterator<Flow> flow_iter = network.getFlows().iterator();
			while ( flow_iter.hasNext() ) {
				Flow flow_of_interest = flow_iter.next();
			
				System.out.println( "Flow of interest : " + flow_of_interest.toString() );
				System.out.println();

//				Analyze the network
//				TFA
				System.out.println( "--- Total Flow Analysis ---" );				
				TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				if ( tf.performAnalysis( flow_of_interest ) ){
					System.out.println( "delay bound     : " + tf.getDelayBound() );
					System.out.println( "backlog bound   : " + tf.getBacklogBound() );
					System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
				} else {
					System.out.println( "TFA analysis failed" );
				}
				
				System.out.println();

//				SFA
				System.out.println( "--- Separated Flow Analysis ---" );
				SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );
				if ( sf.performAnalysis( flow_of_interest ) ){
					System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
					System.out.println( "delay bound     : " + sf.getDelayBound() );
					System.out.println( "backlog bound   : " + sf.getBacklogBound() );
				} else {
					System.out.println( "SFA analysis failed" );
				}
				
				System.out.println();
				
//				PMOOSFA
				System.out.println( "--- PMOO Analysis ---" );
				PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
				if ( pmoo.performAnalysis( flow_of_interest ) ){
					System.out.println( "e2e PMOO SFA SC : " + pmoo.getEndToEndServiceCurve() );
					System.out.println( "delay bound     : " + pmoo.getDelayBound() );
					System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
				} else {
					System.out.println( "PMOO SFA analysis failed" );
				}

				System.out.println();
				System.out.println();
			}
		} catch (Exception e) {
			System.out.println( e.toString() );
		}
	}

	public static void main( String[] args )
	{
		Demo2 demo = new Demo2();
		demo.run();
	}
}
