package unikl.disco.tests;

import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.assertEquals;

import unikl.disco.curves.Curve;
import unikl.disco.misc.Rational;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.PmooSfaAnalysis;

import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Vertex;

public class Node_10Flows_10ACs
{
	static final Curve service_curve = Curve.createRateLatency( 10, 10 ); // latency, rate
	static final Curve max_service_curve = Curve.createRateLatency( 5, 15 ); // latency, rate
	
	static Curve[] arrival_curves = new Curve[10];
	
	static Network network;
	static Vertex v0;
	static Flow f0, f6;
	
	@BeforeClass
	static public void createNetwork()
	{
		network = new Network();
		v0 = network.addVertex( service_curve, max_service_curve );
		v0.setUseGamma( false );
		v0.setUseExtraGamma( false );
	
		try {
			for( int i = 1; i <= 10; i++ ){
				arrival_curves[i-1] = Curve.createTokenBucket( i, i*0.1 ); // burst, rate
				switch( i ){
					case 1: f0 = network.addFlow( arrival_curves[i-1], v0 );
							break;
					case 7: f6 = network.addFlow( arrival_curves[i-1], v0 );
							break;
					default: network.addFlow( arrival_curves[i-1], v0 );
							 break;
				}
			}
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}
	}
	
//--------------------Flow 0--------------------	
	@Test
	public void f0_tfa_fifoMux()
	{
//		TFA: results will be equal for all flows as they share the same path of servers, however, calculations differ due to distinct explicit input nodes
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		v0.setUseFifoMultiplexing( true ); // default = false
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		if ( tf.performAnalysis( f0 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Rational( 15.5 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 110 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_tfa_arbMux()
	{
//		TFA: results will be equal for all flows as they share the same path of servers, however, calculations differ due to distinct explicit input nodes
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		v0.setUseArbitraryMultiplexing( true ); // v0.useArbitraryMultiplexing() != v0.useFifoMultiplexing()
		System.out.println("--- Total Flow Analysis (ARB_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		if ( tf.performAnalysis( f0 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("TFA ARB delay", new Rational( 310, 9 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 110 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		v0.setUseFifoMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		if ( sf.performAnalysis( f0 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("SFA FIFO delay", new Rational( 1796, 115 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 127, 50 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f0_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		v0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (ARB_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		if ( sf.performAnalysis( f0 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("SFA ARB delay", new Rational( 775, 23 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 100, 23 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f0_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		v0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		if ( pmoo.performAnalysis( f0 ) ){
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEndToEndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("PMOO ARB delay", new Rational( 775, 23 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 100, 23 ), pmoo.getBacklogBound() );
	}

//--------------------Flow 6--------------------	
	@Test
	public void f6_tfa_fifoMux()
	{
//		TFA: results will be equal for all flows as they share the same path of servers, however, calculations differ due to distinct explicit input nodes
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		v0.setUseFifoMultiplexing( true ); // default = false
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		if ( tf.performAnalysis( f6 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Rational( 15.5 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 110 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f6_tfa_arbMux()
	{
//		TFA: results will be equal for all flows as they share the same path of servers, however, calculations differ due to distinct explicit input nodes
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		v0.setUseArbitraryMultiplexing( true ); // v0.useArbitraryMultiplexing() != v0.useFifoMultiplexing()
		System.out.println("--- Total Flow Analysis (ARB_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		if ( tf.performAnalysis( f6 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("TFA ARB delay", new Rational( 310, 9 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 110 ), tf.getBacklogBound() );;
	}
	
	@Test
	public void f6_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		v0.setUseFifoMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		if ( sf.performAnalysis( f6 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}

		assertEquals("SFA FIFO delay", new Rational( 2099, 130 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 434, 25 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f6_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		v0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (ARB_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		if ( sf.performAnalysis( f6 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}

		assertEquals("SFA ARB delay", new Rational( 775, 26 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 350, 13 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f6_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		v0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		if ( pmoo.performAnalysis( f6 ) ){
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEndToEndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("PMOO ARB delay", new Rational( 775, 26 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 350, 13 ), pmoo.getBacklogBound() );
	}
}
