package unikl.disco.tests;

import static org.junit.Assert.assertEquals;
import org.junit.BeforeClass;
import org.junit.Test;

import unikl.disco.curves.Curve;
import unikl.disco.misc.Rational;
import unikl.disco.nc.Configuration;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.PmooSfaAnalysis;
import unikl.disco.nc.Configuration.Flag;

import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Vertex;

public class Tandem_1SC_3Flows_1AC_3Paths
{
	static final Curve service_curve = Curve.createRateLatency( 20, 20 ); // latency, rate
	static final Curve arrival_curve = Curve.createTokenBucket( 25, 5 ); // burst, rate
	
	static Network network;
	static Vertex v0, v1, v2;
	static Flow f0, f1, f2;
	
	@BeforeClass
	static public void createNetwork()
	{
		network = new Network();
		v0 = network.addVertex( service_curve );
		v1 = network.addVertex( service_curve );
		v2 = network.addVertex( service_curve );

		try {
			network.addEdge( v0, v1 );
			network.addEdge( v1, v2 );
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}

		try {	
			f0 = network.addFlow( arrival_curve, v0, v1 );
			f1 = network.addFlow( arrival_curve, v2 );
			f2 = network.addFlow( arrival_curve, v0, v2 );
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}
	}
	
//--------------------Flow 0--------------------	
	@Test
	public void f0_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		v0.setUseFifoMultiplexing( true ); // default = true
		v1.setUseFifoMultiplexing( true ); // default = true
		v2.setUseFifoMultiplexing( true ); // default = true
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		if ( tf.performAnalysis( f0 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Rational( 55 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 450 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		v0.setUseArbitraryMultiplexing( true ); // default = true
		v1.setUseArbitraryMultiplexing( true ); // default = true
		v2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		if ( tf.performAnalysis( f0 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}

		assertEquals("TFA ARB delay", new Rational( 110 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 450 ), tf.getBacklogBound() );
	}

	@Test
	public void f0_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		if ( sf.performAnalysis( f0 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.NODELOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Rational( 295, 6 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 262.5 ), sf.getBacklogBound() );
	}

	@Test
	public void f0_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		v0.setUseArbitraryMultiplexing( true ); // default = true
		v1.setUseArbitraryMultiplexing( true ); // default = true
		v2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		if ( sf.performAnalysis( f0 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("SFA ARB delay", new Rational( 65 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 1025, 3 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f0_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		v0.setUseArbitraryMultiplexing( true ); // default = true
		v1.setUseArbitraryMultiplexing( true ); // default = true
		v2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		if ( pmoo.performAnalysis( f0 ) ){
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEndToEndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println();
		}

		assertEquals("PMOO ARB delay", new Rational( 170, 3 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 300 ), pmoo.getBacklogBound() );
	}

//--------------------Flow 1--------------------	
	@Test
	public void f1_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		v0.setUseFifoMultiplexing( true ); // default = true
		v1.setUseFifoMultiplexing( true ); // default = true
		v2.setUseFifoMultiplexing( true ); // default = true
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		if ( tf.performAnalysis( f1 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "server alpha map: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Rational( 2205, 64 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 7825, 16 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f1_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		v0.setUseArbitraryMultiplexing( true ); // default = true
		v1.setUseArbitraryMultiplexing( true ); // default = true
		v2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		if ( tf.performAnalysis( f1 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "server alpha map: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("TFA ARB delay", new Rational( 1405, 18 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 5225, 9 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f1_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		if ( sf.performAnalysis( f1 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println( "cross per server: " + sf.getServerCrossTrafficAlphaMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.NODELOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Rational( 6695, 192 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 12225, 64 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f1_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		v0.setUseArbitraryMultiplexing( true ); // default = true
		v1.setUseArbitraryMultiplexing( true ); // default = true
		v2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );
		
		if ( sf.performAnalysis( f1 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println( "cross per server: " + sf.getServerCrossTrafficAlphaMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("SFA ARB delay", new Rational( 1405, 27 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 7475, 27 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f1_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		v0.setUseArbitraryMultiplexing( true ); // default = true
		v1.setUseArbitraryMultiplexing( true ); // default = true
		v2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		if ( pmoo.performAnalysis( f1 ) ){
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEndToEndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("PMOO ARB delay", new Rational( 145, 3 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 775, 3 ), pmoo.getBacklogBound() );
	}
//--------------------Flow 2--------------------	
	@Test
	public void f2_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		v0.setUseFifoMultiplexing( true ); // default = true
		v1.setUseFifoMultiplexing( true ); // default = true
		v2.setUseFifoMultiplexing( true ); // default = true
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		if ( tf.performAnalysis( f2 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "cross per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Rational( 5725, 64 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 7825, 16 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f2_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		v0.setUseArbitraryMultiplexing( true ); // default = true
		v1.setUseArbitraryMultiplexing( true ); // default = true
		v2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		if ( tf.performAnalysis( f2 ) ){
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "cross per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} else {
			System.out.println( "TFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("TFA ARB delay", new Rational( 16925, 90 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 5225, 9 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f2_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		if ( sf.performAnalysis( f2 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.NODELOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Rational( 845, 12 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 1475, 4 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f2_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		v0.setUseArbitraryMultiplexing( true ); // default = true
		v1.setUseArbitraryMultiplexing( true ); // default = true
		v2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		if ( sf.performAnalysis( f2 ) ){
			System.out.println( "e2e SFA SC      : " + sf.getEndToEndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "SFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("SFA ARB delay", new Rational( 280, 3 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 1450, 3 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f2_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		v0.setUseArbitraryMultiplexing( true ); // default = true
		v1.setUseArbitraryMultiplexing( true ); // default = true
		v2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		if ( pmoo.performAnalysis( f2 ) ){
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEndToEndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} else {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println();
		}
		
		assertEquals("PMOO ARB delay", new Rational( 85 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 1325, 3 ), pmoo.getBacklogBound() );
	}
}